package com.caimei365.commodity.components;


import com.caimei365.commodity.mapper.PriceMapper;
import com.caimei365.commodity.mapper.PromotionsMapper;
import com.caimei365.commodity.model.dto.Sku;
import com.caimei365.commodity.model.vo.*;
import com.caimei365.commodity.utils.ImageUtils;
import com.caimei365.commodity.utils.MathUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * Description
 *
 * @author : Charles
 * @date : 2020/6/22
 */
@Slf4j
@Service
public class PriceUtilService {
    @Resource
    private PriceMapper priceMapper;
    @Resource
    private PromotionsMapper promotionsMapper;
    @Value("${caimei.wwwDomain}")
    private String domain;

    /**
     * 根据价格返回价格等级
     *
     * @param price 价格
     * @return priceGrade 价格等级
     */
    public Integer getPriceGrade(Double price) {
        int grade = 1;
        price = price != null ? price : 0d;
        if (price > 50 * 10000) {
            grade = 5;
        } else if (price > 25 * 10000) {
            grade = 4;
        } else if (price > 10 * 10000) {
            grade = 3;
        } else if (price > 2.5 * 10000) {
            grade = 2;
        }
        return grade;
    }

    /**
     * 设置阶梯价展示数据
     *
     * @param ladderPrices List<LadderPriceVo>
     * @param tax          TaxVo
     */
    public void setLadderPriceList(List<LadderPriceVo> ladderPrices, TaxVo tax) {
        IntStream.range(0, ladderPrices.size()).forEach(i -> {
            //税费标志
            boolean taxFlag = 0 == tax.getIncludedTax() && 1 == tax.getInvoiceType() || 2 == tax.getInvoiceType();
            //添加税费
            if (taxFlag) {
                BigDecimal taxFee = MathUtil.div(MathUtil.mul(ladderPrices.get(i).getBuyPrice(), tax.getTaxPoint()), 100, 2);
                ladderPrices.get(i).setBuyPrice(MathUtil.add(ladderPrices.get(i).getBuyPrice(), taxFee).doubleValue());
            }
            if (i == ladderPrices.size() - 1) {
                ladderPrices.get(i).setMaxNum(0);
                ladderPrices.get(i).setNumRange("≥" + ladderPrices.get(i).getBuyNum());
            } else {
                ladderPrices.get(i).setMaxNum(ladderPrices.get(i + 1).getBuyNum());
                String buyNumRangeShow = ladderPrices.get(i).getBuyNum() + "~" + (ladderPrices.get(i + 1).getBuyNum() - 1);
                ladderPrices.get(i).setNumRange(buyNumRangeShow);
            }
        });
    }

    /**
     * 根据用户id设置详细价格
     *
     * @param price  商品价格类
     * @param userId 用户Id
     */
    public void setPriceByUserId(PriceVo price, Integer userId) {
        // 根据用户Id查询用户身份: 0个人，1协销，2会员机构，3供应商，4普通机构
        Integer identity = priceMapper.getIdentityByUserId(userId);
        if (null == identity) {
            identity = 0;
        }
        // priceFlag: 0公开价格，1不公开价格，2仅对会员机构公开，3仅对医美机构公开
        Integer priceFlag = null != price.getPriceFlag() ? price.getPriceFlag() : 0;
        // 会员机构类型：1医美，2生美
        Integer clubType = 0;
        if (identity == 2) {
            clubType = priceMapper.getClubTypeById(userId);
            if (null == clubType) {
                clubType = 0;
            }
        }
        // 根据用户Id查询超级会员id
        Integer svipUserId = priceMapper.getSvipUserIdByUserId(userId);
        // 超级会员标识
        boolean isSuperVip = null != svipUserId;
        // 根据商品id，用户身份 查询商品活动
        PromotionsVo promotions = promotionsMapper.getPricePromotions(price.getProductId(), identity);
        price.setActStatus(0);
        // 用户身份: 2-会员机构, 4-普通机构
        price.setUserIdentity(identity);
        //是否是供应商自己的商品
        boolean isShopOwn = false;
        if (identity == 3) {
            // 根据用户Id查询供应商Id
            Integer shopId = priceMapper.getShopIdByUserId(userId);
            if (null != shopId && shopId.equals(price.getShopId())) {
                isShopOwn = true;
            }
        }
        // 设置价格标志(协销|价格公开|仅对医美机构公开|仅对会员机构公开|供应商自己的商品)
        boolean setFlag = identity == 1 || priceFlag == 0 || (priceFlag == 3 && clubType == 1) || (priceFlag == 2 && (identity == 2 || isSuperVip)) || isShopOwn;
        if (setFlag) {
            List<Sku> skus = price.getSkus();
            if (null == skus || skus.size() < 1) {
                List<Sku> skuList = priceMapper.findSkus(price.getProductId());
                price.setSkus(skuList);
                skus = skuList;
            }
            //给一个默认的最低价和最低起订量
            price.setPrice(skus.get(0).getPrice());
            price.setSkuId(skus.get(0).getSkuId());
            price.setMinBuyNumber(skus.get(0).getMinBuyNumber());
            skus.forEach(s -> {
                // 设置划线价
                s.setOriginalPrice(s.getPrice());
                //税费标志
                boolean taxFlag = "0".equals(price.getIncludedTax()) && ("1".equals(price.getInvoiceType()) || "2".equals(price.getInvoiceType()));
                if (1 == price.getSvipProductFlag()) {
                    // 超级会员优惠商品,不参与促销活动（普通机构购买该商品可参与店铺促销）/阶梯价/复购价
                    s.setLadderPriceFlag(0);
                    // 超级会员优惠价
                    BigDecimal svipDiscountPrice = price.getSvipDiscountPrice();
                    if (1 == price.getSvipPriceType()) {
                        // 折扣价
                        price.setSvipPriceTag(MathUtil.div(price.getSvipDiscount(), 10, 1) + "折");
                        if (isSuperVip) {
                            // 超级会员用户设置优惠价
                            svipDiscountPrice = MathUtil.div(MathUtil.mul(s.getPrice(), price.getSvipDiscount()), 100, 2);
                            s.setPrice(svipDiscountPrice.doubleValue());
                        }
                    } else if (2 == price.getSvipPriceType()) {
                        // 直接优惠价
                        if (isSuperVip) {
                            // 超级会员用户设置优惠价
                            s.setPrice(svipDiscountPrice.doubleValue());
                        }
                        if (taxFlag) {
                            //添加税费
                            BigDecimal thisTaxFee = MathUtil.div(MathUtil.mul(svipDiscountPrice, price.getTaxRate()), 100, 2);
                            svipDiscountPrice = MathUtil.add(svipDiscountPrice, thisTaxFee);
                        }
                        price.setSvipPriceTag("￥" + svipDiscountPrice);
                    }
                } else if (null != promotions) {
                    // 促销活动
                    price.setActStatus(1);
                    s.setLadderPriceFlag(0);
                    if (promotions.getMode() == 3) {
                        // 获取赠品
                        List<ProductItemVo> giftList = promotionsMapper.getPromotionGifts(promotions.getId());
                        promotions.setGiftList(giftList);
                    }
                    if (promotions.getType() == 1 && promotions.getMode() == 1 && null != promotions.getTouchPrice()) {
                        s.setPrice(promotions.getTouchPrice());
                    }
                    price.setPromotions(promotions);
                } else {
                    if (null != userId) {
                        if (1 == s.getLadderPriceFlag()) {
                            // 阶梯价
                            LadderPriceVo lowerPrice = priceMapper.findLowerLadderPrice(s.getSkuId());
                            LadderPriceVo lowerBuyNum = priceMapper.findMaxLadderPrice(s.getSkuId());
                            if (null != lowerPrice) {
                                if (null != lowerBuyNum) {
                                    s.setMinBuyNumber(lowerBuyNum.getBuyNum());
                                    s.setPrice(lowerBuyNum.getBuyPrice());
                                }
                            } else {
                                s.setLadderPriceFlag(0);
                            }
                            List<LadderPriceVo> ladderPrices = priceMapper.getLadderPricesByProductId(s.getSkuId());
                            TaxVo tax = priceMapper.getTaxByProductId(price.getProductId());
                            if (!CollectionUtils.isEmpty(ladderPrices)) {
                                setLadderPriceList(ladderPrices, tax);
                            }
                            s.setLadderPriceList(ladderPrices);
                            price.setLadderPriceFlag(1);
                        } else {
                            price.setLadderPriceFlag(0);
                            // 复购价
                            Double repurchase = priceMapper.getRepurchasePrice(s.getSkuId(), userId);
                            if (null != repurchase && repurchase > 0) {
                                s.setPrice(repurchase);
                                s.setRepurchaseFlag(1);
                                s.setLadderPriceFlag(0);
                            }
                        }
                    }
                }
                //添加税费
                if (taxFlag) {
                    BigDecimal thisTaxFee = MathUtil.div(MathUtil.mul(s.getPrice(), price.getTaxRate()), 100, 2);
                    s.setPrice(MathUtil.add(s.getPrice(), thisTaxFee).doubleValue());
                    BigDecimal originalTaxFee = MathUtil.div(MathUtil.mul(s.getOriginalPrice(), price.getTaxRate()), 100, 2);
                    s.setOriginalPrice(MathUtil.add(s.getOriginalPrice(), originalTaxFee).doubleValue());
                }
            });
            price.setSkus(skus);
        } else {
            price.setUserIdentity(0);
            if (null != promotions) {
                price.setActStatus(1);
                promotions.setTouchPrice(0d);
                promotions.setReducedPrice(0d);
                price.setPromotions(promotions);
            } else {
                // 默认非促销活动状态
                price.setActStatus(0);
            }
        }
    }

    /**
     * 设置商品主图及价格
     *
     * @param userId  用户Id
     * @param product ProductItemVo
     */
    public void setProductDetails(Integer userId, ProductItemVo product) {
        // 设置图片
        product.setImage(ImageUtils.getImageURL("product", product.getImage(), 0, domain));
        // 设置sku
        List<Sku> skus = priceMapper.findSkus(product.getProductId());
        AtomicBoolean lowPrice = new AtomicBoolean(false);
        skus.forEach(s -> {
            // 设置价格等级
            s.setPriceGrade(getPriceGrade(s.getPrice()));
            s.setOriginalPrice(s.getPrice());
            if (1 == s.getLadderPriceFlag()) {
                List<LadderPriceVo> ladderPrices = priceMapper.getLadderPricesByProductId(s.getSkuId());
                TaxVo tax = priceMapper.getTaxByProductId(product.getProductId());
                if (!CollectionUtils.isEmpty(ladderPrices)) {
                    setLadderPriceList(ladderPrices, tax);
                }
                s.setLadderPriceList(ladderPrices);
                lowPrice.set(true);
            }
        });
        //设置展示的默认最低价和展示最低价的规格
        if (lowPrice.get()) {
            ProductItemVo showProduct = priceMapper.findLowPrice(product.getProductId());
            product.setSkuId(showProduct.getSkuId());
            product.setPrice(showProduct.getPrice());
            product.setOriginalPrice(showProduct.getPrice());
            product.setUnit(showProduct.getUnit());
            product.setMinBuyNumber(showProduct.getMinBuyNumber());
            product.setPriceGrade(getPriceGrade(showProduct.getPrice()));
        } else {
            List<Sku> collect = skus.stream().sorted(Comparator.comparing(Sku::getPrice)).collect(Collectors.toList());
            product.setSkuId(collect.get(0).getSkuId());
            product.setPrice(collect.get(0).getPrice());
            product.setOriginalPrice(collect.get(0).getPrice());
            product.setUnit(collect.get(0).getUnit());
            product.setMinBuyNumber(collect.get(0).getMinBuyNumber());
            product.setPriceGrade(getPriceGrade(collect.get(0).getPrice()));
        }
        product.setSkus(skus);
        // 数据库获取基本价格信息
        PriceVo price = priceMapper.getDetailPrice(product.getProductId());
        price.setSkus(skus);
        // 根据用户id设置详细价格
        setPriceByUserId(price, userId);
        // 设置价格
        product.setActStatus(price.getActStatus());
        product.setPromotions(price.getPromotions());
        product.setUserIdentity(price.getUserIdentity());
        product.setSvipProductFlag(price.getSvipProductFlag());
        product.setSvipPriceTag(price.getSvipPriceTag());
    }

}
